@@ -113,9 +113,14 @@ def print_array_info(ar):
 
 def output_array_stats(ar, name, verbose=True):
     ar = nm.asarray(ar)
-    output('%s\nmin: %.7e mean: %.7e median: %.7e max: %.7e'
-           % (name, ar.min(), ar.mean(), nm.median(ar), ar.max()),
-           verbose=verbose)
+    if nm.isrealobj(ar):
+        output('%s\nmin: % .6e mean: % .6e median: % .6e max: % .6e'
+               % (name, ar.min(), ar.mean(), nm.median(ar), ar.max()),
+               verbose=verbose)
+
+    else:
+        output_array_stats(ar.real, 'Re(%s)' % name , verbose=verbose)
+        output_array_stats(ar.imag, 'Im(%s)' % name, verbose=verbose)
 
 ##
 # 21.11.2005, c
