@@ -25,7 +25,7 @@ def mumps_parallel_solve():
         n = flags[0]
         mumps_ls.set_rcd_centralized(idxs[0, :], idxs[1, :], vals_mtx, n)
         x = vals_rhs.copy()
-        mumps_ls.set_b(x)
+        mumps_ls.set_rhs(x)
 
     mumps_ls(6)  # analyse, factorize, solve
 
@@ -37,6 +37,7 @@ def mumps_parallel_solve():
     del(mumps_ls)
 
 
-comm = MPI.Comm.Get_parent()
-mumps_parallel_solve()
-comm.Disconnect()
+if __name__ == '__main__':
+    comm = MPI.Comm.Get_parent()
+    mumps_parallel_solve()
+    comm.Disconnect()
